/***********************************************************************
**
**  REBOL Invasion 3.0
**
**  ext-test.c - Example REBOL Extension test
**
***********************************************************************/

#define ODD_INT_64

#include "reb-c.h"
#include "reb-ext.h"

#include "ext-data.h"

u32 *word_ids = 0; // used to hold word identifiers

RXIEXT const char *RX_Init(int opts, RXILIB *lib) {
	RXI = lib;
	if (sizeof(RXIARG) != 8 || lib->version != RXI_VERSION)
		return 0;
	return init_block;
}

RXIEXT int RX_Quit(int opts) {
	return 0;
}

RXIEXT int RX_Call(int cmd, RXIFRM *frm) {
	switch (cmd) {

	case CMD_MAP_WORDS:
		word_ids = RXI_MAP_WORDS(RXA_SERIES(frm, 1));
		return RXR_TRUE;

	case CMD_T_UNSET:
		return RXR_UNSET;

	case CMD_T_NONE:
		return RXR_NONE;

	case CMD_T_TRUE:
		return RXR_TRUE;

	case CMD_T_FALSE:
		return RXR_FALSE;

	case CMD_T_VALUE:
		return RXR_VALUE; // return first arg

	case CMD_T_BLOCK:
		return RXR_BLOCK; // 3 args become a block

	case CMD_T_WORD:
		RXA_INT64(frm, 1) = RXA_WORD(frm, 1);
		RXA_TYPE(frm, 1) = RXT_INTEGER;
		break;

	case CMD_T_WORD_MAP:
		RXA_INT64(frm, 1) = RXI_FIND_WORD(word_ids, RXA_WORD(frm, 1));
		RXA_TYPE(frm, 1) = RXT_INTEGER;
		break;

	case CMD_T_NEG:
		RXA_INT64(frm, 1) = -RXA_INT64(frm, 1);
		break;

	case CMD_T_ADDI:
		RXA_INT64(frm, 1) = RXA_INT64(frm, 1) + RXA_INT64(frm, 2);
		break;

	case CMD_T_ADDD:
		RXA_DEC64(frm, 1) = RXA_DEC64(frm, 1) + RXA_DEC64(frm, 2);
		break;

	case CMD_T_SER_LEN:
		RXA_INT64(frm, 1) = RXI_SERIES_INFO(RXA_SERIES(frm, 1), RXI_INFO_TAIL);
		RXA_TYPE(frm, 1) = RXT_INTEGER;
		break;

	case CMD_T_SER_SIZE:
		RXA_INT64(frm, 1) = RXI_SERIES_INFO(RXA_SERIES(frm, 1), RXI_INFO_SIZE);
		RXA_TYPE(frm, 1) = RXT_INTEGER;
		break;

	case CMD_T_PICKS:
		RXA_INT64(frm, 1) = RXI_GET_CHAR(RXA_SERIES(frm, 1), (u32)RXA_INT64(frm, 2)-1);
		RXA_TYPE(frm, 1) = RXT_CHAR;
		break;

	case CMD_T_POKES:
		RXI_SET_CHAR(RXA_SERIES(frm, 1), (u32)RXA_INT64(frm,2)-1, RXA_CHAR(frm, 3));
		break;

	case CMD_T_PICKB:
		RXA_TYPE(frm, 1) = RXI_GET_VALUE(RXA_SERIES(frm, 1), (u32)RXA_INT64(frm, 2)-1, &RXA_ARG(frm, 1));
		break;

	case CMD_T_POKEB:
		RXI_SET_VALUE(RXA_SERIES(frm, 1), (u32)RXA_INT64(frm,2)-1, RXA_ARG(frm, 3), RXA_TYPE(frm, 3));
		break;

	case CMD_T_HANDLE:
		RXA_HANDLE(frm, 1) = word_ids;
		RXA_TYPE(frm, 1) = RXT_HANDLE;
		break;

	case CMD_T_HANDLEQ:
		return (RXA_HANDLE(frm, 1) == word_ids) ? RXR_TRUE : RXR_FALSE;

	case CMD_T_MAKES:
		{
			REBSER *s = RXI_MAKE_STRING(100, 0);
			RXI_SET_CHAR(s, 0, 'A');
			RXI_SET_CHAR(s, 1, 'B');
			RXI_SET_CHAR(s, 2, 'C');
			RXA_SERIES(frm, 1) = s;
			RXA_INDEX(frm, 1) = 0;
			RXA_TYPE(frm, 1) = RXT_STRING;
		}
		break;

	case CMD_T_MAKEB:
		{
			REBSER *s = RXI_MAKE_BLOCK(10);
			RXIARG v;
			v.int32a = 1;
			RXI_SET_VALUE(s, 0, v, RXT_LOGIC);
			v.int64 = 1;
			RXI_SET_VALUE(s, 1, v, RXT_INTEGER);
			v.dec64 = 1.2;
			RXI_SET_VALUE(s, 2, v, RXT_DECIMAL);
			RXA_SERIES(frm, 1) = s;
			RXA_INDEX(frm, 1) = 0;
			RXA_TYPE(frm, 1) = RXT_BLOCK;
		}
		break;

	default:
		return RXR_NO_COMMAND;
	}

	return RXR_VALUE;
}
